@@ -103,16 +103,25 @@ sub _parse_item {
   my $item;
   if (! ref $_[0]) {
     eval { $item = SILK_IPADDR_CLASS->new($_[0]) };
+    if ($@) {
+      eval { $item = SILK_CIDR_CLASS->new($_[0]) };
+    }
+    if ($@) {
+      eval { $item = SILK_RANGE_CLASS->new($_[0]) };
+    }
     if ($@) {
       eval { $item = SILK_IPWILDCARD_CLASS->new($_[0]) };
     }
   }
   elsif (UNIVERSAL::isa($_[0], 'ARRAY')) {
-    $item = __PACKAGE__->new(@{$_[0]});
-  }
-  else {
-    # SILK_IPRANGE_CLASS or SILK_IPCIDR_CLASS
-    eval { $item = $_[0]->as_str };
+    if (UNIVERSAL::isa($_[0], SILK_CIDR_CLASS) ||
+        UNIVERSAL::isa($_[0], SILK_RANGE_CLASS)) {
+      $item = $_[0];
+    }
+    else {
+      # flatten the array
+      $item = __PACKAGE__->new(@{$_[0]});
+    }
   }
   $item ||= $_[0];
 }
@@ -168,6 +177,12 @@ sub add {
     if (UNIVERSAL::isa($_, SILK_IPADDR_CLASS)) {
       Net::Silk::IPSet::add_addr($self, $_);
     }
+    elsif (UNIVERSAL::isa($_, SILK_CIDR_CLASS)) {
+      $self->add_cidr(@$_);
+    }
+    elsif (UNIVERSAL::isa($_, SILK_RANGE_CLASS)) {
+      $self->add_range(@$_);
+    }
     elsif (UNIVERSAL::isa($_, SILK_IPWILDCARD_CLASS)) {
       Net::Silk::IPSet::add_wildcard($self, $_);
     }
@@ -175,11 +190,7 @@ sub add {
       Net::Silk::IPSet::_union_update($self, $_);
     }
     else {
-      eval { $self->add_cidr($_) };
-      next unless $@;
-      eval { $self->add_range($_) };
-      next unless $@;
-      croak "ip address, string, number, wildcard, or ipset required: $_"
+      croak "ip address, string, cidr, range, wildcard, or ipset required: $_"
     }
   }
 }
@@ -188,11 +199,10 @@ sub add_cidr {
   my $self = shift;
   my($addr, $prefix) = @_;
   if (! defined $prefix) {
-    if (m{^\s*([^/]+)/(\d+)\s*$}) {
-      ($addr, $prefix) = ($1, $2);
-    }
-    else {
-      $prefix = 32;
+    eval { ($addr, $prefix) = ($addr->ip, $addr->bits) };
+    if ($@) {
+      my $cidr = SILK_CIDR_CLASS->new($addr);
+      ($addr, $prefix) = @$cidr;
     }
   }
   Net::Silk::IPSet::add_addr($self, $addr, $prefix);
@@ -202,11 +212,10 @@ sub add_range {
   my $self = shift;
   my($lo, $hi) = @_;
   if (! defined $hi) {
-    if ($lo =~ tr/-/-/ == 1) {
-      ($lo, $hi) = split(/-/, $lo);
-    }
-    else {
-      $hi = $lo;
+    eval { ($lo, $hi) = ($lo->first, $lo->last) };
+    if ($@) {
+      my $range = SILK_RANGE_CLASS->new($lo);
+      ($lo, $hi) = @$range;
     }
   }
   Net::Silk::IPSet::_add_range($self, $lo, $hi);
@@ -670,12 +679,12 @@ Net::Silk::IPSet - SiLK IP sets
 
   $iter = $s3->iter_cidr;
   while ($cidr = $iter->()) {
-    print join('/', @$cidr), "\n";
+    print "$cidr\n";
   }
 
-  $iter = $s3->iter_cidr_str;
-  while ($cidr = $iter->()) {
-    print "$cidr\n";
+  $iter = $s3->iter_ranges;
+  while ($range = $iter->()) {
+    print "$range\n";
   }
 
   $s3->clear;
@@ -694,8 +703,8 @@ The I<items> can be any of the following:
   * Net::Silk::IPAddr or string representation
   * Net::Silk::IPWildcard or string representation
   * Net::Silk::IPSet
-  * string representation of a CIDR block (x.x.x.x/n)
-  * string representation of a range (x.x.x.x-y.y.y.y)
+  * Net::Silk::Range or string representation of a range (x.x.x.x-y.y.y.y)
+  * Net::Silk::CIDR string representation of a CIDR block (x.x.x.x/n)
   * ref to array of any of the above
   * arbitrary list of any of the above (multiple arguments)
 
@@ -919,9 +928,9 @@ C<Net::Silk::IPSet> objects:
 
 =head1 TIED HASH
 
-The IPSet object reference can be treated like a hash reference,
-with each key being an IP address with a value of 1. So containment
-can be tested with C<if ($ipset->{$ip}) { ... }>
+The IPSet object reference can be treated like a hash reference, with
+each key being an IP address with a value of 1. So containment can be
+tested with C<if ($ipset->{$ip}) { ... }>
 
 =head1 SEE ALSO
 
