@@ -57,8 +57,7 @@ use overload (
 );
 
 use Net::Silk qw( :basic );
-use Net::Silk::Site qw( HAVE_SITE_CONFIG_SILENT );
-use Net::Silk::IPAddr;
+use Net::Silk::Site;
 
 use DateTime;
 use DateTime::Duration;
@@ -130,7 +129,7 @@ sub as_hash {
                  uniform_packets )) {
     $r{$m} = $self->$m;
   }
-  if (HAVE_SITE_CONFIG_SILENT()) {
+  if (Net::Silk::Site::HAVE_SITE_CONFIG_SILENT()) {
     for my $m (qw( classtype_id classtype sensor_id sensor )) {
       $r{$m} = $self->$m;
     }
@@ -216,7 +215,7 @@ sub protocol {
 
 sub sensor {
   my $self = shift;
-  return unless HAVE_SITE_CONFIG_SILENT();
+  return unless Net::Silk::Site::HAVE_SITE_CONFIG_SILENT();
   if (@_) {
     my $sensor = shift;
     if (! looks_like_number($sensor)) {
@@ -237,19 +236,19 @@ sub sensor_id {
 
 sub classname {
   my $self = shift;
-  return unless HAVE_SITE_CONFIG_SILENT();
+  return unless Net::Silk::Site::HAVE_SITE_CONFIG_SILENT();
   $self->_classname();
 }
 
 sub typename {
   my $self = shift;
-  return unless HAVE_SITE_CONFIG_SILENT();
+  return unless Net::Silk::Site::HAVE_SITE_CONFIG_SILENT();
   $self->_typename();
 }
 
 sub classtype {
   my $self = shift;
-  return unless HAVE_SITE_CONFIG_SILENT();
+  return unless Net::Silk::Site::HAVE_SITE_CONFIG_SILENT();
   if (@_) {
     my $classtype;
     if (@_ > 1 || ref $_[0]) {
